import click

import extractGeoDataFromFolder as fext
import getBoundingBox as box
import getPolygon as poly
import getTimeExtent as timeext


# asking for parameters in command line
@click.command()
@click.option('--path', prompt="Please enter path to Folder", help='Path to Folder containing Geofiles')
@click.option('--clear','-c', default=False, is_flag=True, help='Clear screen before showing results')
@click.option('--time', '-t', default=False, is_flag=True, help="execute time extraction for one file")
@click.option('--space', '-s', default=False, is_flag=True, help="execute boundingbox extraction for one file")
@click.option('--hull', '-h', default=False, is_flag=True, help="execute convex-hull extraction for one file")
def main(path, clear, time, space, hull):
    """Main CLI-Tool combining all the single extraction tools in one tool.
    @param path     Text/Path: Path to the folder with Geodata
    @param clear    Boolean: Whether to clear the screen before showing the result or not
    @param time     Boolean: If true, time extraction will be triggered
    @param space    Boolean: If true, spatial extraction will be triggered
    @param hull     Boolean: If true, a convex hull will be calculated

    No return but Console-Output of the wanted extents. If time, space and hull are all false, the folder extraction will be triggered and returns a table with the boundingbox and spatial extent for each individual file in addition to the full time and spatial extent of he folder.
    Issue: If there are files that are not in WGS84 and do not have a specified referencesystem, the spatial extent of the folder will might not be in WGS84
    """
    output = []

    name = ""
    if time or space or hull:
        name = click.prompt("Please enter filename")

    def timeOption(path, name):
        res = timeext.getTimeExtent(name, path)
        if res[0] is not None:
            return res[0]
        else:
            return res[1]

    def spaceOption(path, name):
        res = box.getBoundingBox(name, path)
        if res[0] is not None:
            return res[0]
        else:
            return res[1]

    def polyOption(path, name):
        res = poly.getPolygon(name, path)
        if res[0] is not None:
            return res[0]
        else:
            return res[1]

#################################################################

    if time:
        output.append("Timeextent:")
        output.append(timeOption(path, name))
        output.append("\n")
    
    if space:
        output.append("Spatialextent:")
        output.append(spaceOption(path, name))
        output.append("\n")

    if hull:
        output.append("Spatialextent as Convex Hull:")
        output.append(polyOption(path, name))
        output.append("\n")

    if not (time or space or hull):
        fext.extractFromFolder(path, clear)
    else:
        if clear:
            click.clear()
        for x in output:
            click.echo(x)

if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="geodataExtent",
    version="0.1.1",
    author="A^sHL^2",
    author_email="h_fock01@uni-muenster.de",
    description="Package for extracting the time and spatial extent of Geodata",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/HenFo/Geosoftware-II-AALLH/tree/master/CLI%20Tools",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "gdal >= 2.3.0",
        "click >= 7.0",
        "netCDF4 >= 1.4.2",
        "pandas >= 0.23.4",
        "pygeoj >= 1.0.0",
        "pyshp >= 2.0.0",
        "xarray >= 0.11.0",
        "DateTime >= 4.3",
        "tabulate >= 0.8.2",
    ],
    entry_points = {
        'console_scripts': ['extract-extent=geodataExtent.masterExtract:main'],
    }
)


"""
@Author Henry Fock
"""
import sys
# add local modules folder
file_path = '../Python_Modules'
sys.path.append(file_path)

import getBoundingBox as box
import getTimeExtent as timeEx
import subprocess
import threading

from os import listdir
from os.path import isfile, join
from osgeo import ogr
import click
from DateTime import DateTime
from tabulate import tabulate

@click.command()
@click.option('--path', prompt="Path to folder", help='Path to your folder as String')
@click.option('--clear', default=False, help='Flag wether you want to display only the Output')
def main(path, clear):
    extractFromFolder(path, clear)

def extractFromFolder(path, clear):
    threads = []
    res = []
    onlyfiles = [f for f in listdir(path) if isfile(join(path, f))]

    for name in onlyfiles:
        current = CliTools(path, name)
        threads.append(current)
        current.start()

    for thread in threads:
        thread.join()
        res.append(thread.result)

    if clear:
        click.clear()

    output = []
    click.secho("\nResult:" , blink=True)
    for x in res:
        if x[1][0] is None:
            if x[2][0] is None:
                output.append([x[0], x[1][1], x[2][1]])
            else:
                output.append([x[0], x[1][1], x[2][0]])
        else:
            if x[2][0] is None:
                output.append([x[0], x[1][0], x[2][1]])
            else:
                output.append([x[0], x[1][0], x[2][0]])

    click.echo(tabulate(output, headers=['Filename', 'Boundingbox (minX/Lon, minY/Lat, maxX, maxY) / Error', 'Timeextent (stard, end, Average-interval in days) / Error']))

    # Create a geometry collection
    geomcol =  ogr.Geometry(ogr.wkbGeometryCollection)
    lowTime, maxTime = None, None 

    for x in res:
        if x[1][1] is None:
            box = ogr.Geometry(ogr.wkbLineString)
            box.AddPoint(x[1][0][0],x[1][0][1])
            box.AddPoint(x[1][0][0], x[1][0][3])
            box.AddPoint(x[1][0][2],x[1][0][1])
            box.AddPoint(x[1][0][2], x[1][0][3])
            geomcol.AddGeometry(box)
        
        if x[2][1] is None:
            if lowTime is None or DateTime(x[2][0][0]) < lowTime:
                lowTime = DateTime(x[2][0][0])
            
            if maxTime is None or DateTime(x[2][0][1]) > maxTime:
                maxTime = DateTime(x[2][0][1])
    
    env = geomcol.GetEnvelope()
    click.echo("\n")
    click.echo(tabulate([[str((env[0], env[2], env[1], env[3])).strip('()'), str((str(lowTime), str(maxTime))).strip('()')]], headers=['Boundingbox of Folder', 'Timeextent of Folder (ISO8601)']))
    click.echo("\n")

    # click.echo("\nFull spatial Extent as Boundingbox: %s" % str((env[0], env[2], env[1], env[3])))
    # click.echo("Full time Extend as ISO8601: %s" % str((str(lowTime), str(maxTime))))
    # click.echo("\n")




class CliTools(threading.Thread):
    def __init__(self, path, name): 
        threading.Thread.__init__(self) 
        self.path = path
        self.name = name
        self.result = []

    def run(self):
        bbox = box.getBoundingBox(path=self.path, name=self.name)
        time = timeEx.getTimeExtent(path=self.path, name=self.name)
        self.result.extend([self.name, bbox, time])


if __name__ == '__main__':
    main()